# Generated by Django 4.2.11 on 2024-04-09 10:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("plans", "0012_planpricing_visible"),
    ]

    operations = [
        migrations.AlterField(
            model_name="recurringuserplan",
            name="has_automatic_renewal",
            field=models.BooleanField(
                db_column="has_automatic_renewal",
                default=False,
                help_text="Automatic renewal is enabled for associated plan. If False, the plan renewal can be still initiated by user.",
                verbose_name="has automatic plan renewal",
            ),
        ),
        migrations.RenameField(
            model_name="recurringuserplan",
            old_name="has_automatic_renewal",
            new_name="_has_automatic_renewal_backup_deprecated",
        ),
        migrations.AddField(
            model_name="recurringuserplan",
            name="renewal_triggered_by",
            field=models.IntegerField(
                choices=[(1, "other"), (2, "user"), (3, "task")],
                db_index=True,
                default=2,
                help_text="The source of the associated plan's renewal (USER = user-initiated renewal, TASK = autorenew_account-task-initiated renewal, OTHER = renewal is triggered using another mechanism).",
                verbose_name="renewal triggered by",
            ),
        ),
    ]
