# Generated by Django 3.2.11 on 2022-02-08 12:44

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("plans", "0010_auto_20220113_1317"),
    ]

    operations = [
        migrations.AddField(
            model_name="billinginfo",
            name="created",
            field=models.DateTimeField(
                auto_now_add=True,
                db_index=True,
                default=None,
                null=True,
                verbose_name="created",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="billinginfo",
            name="updated_at",
            field=models.DateTimeField(auto_now=True, null=True),
        ),
        migrations.AddField(
            model_name="invoice",
            name="created",
            field=models.DateTimeField(
                auto_now_add=True,
                db_index=True,
                default=None,
                null=True,
                verbose_name="created",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="invoice",
            name="updated_at",
            field=models.DateTimeField(auto_now=True, null=True),
        ),
        migrations.AddField(
            model_name="order",
            name="updated_at",
            field=models.DateTimeField(auto_now=True, null=True),
        ),
        migrations.AddField(
            model_name="plan",
            name="updated_at",
            field=models.DateTimeField(auto_now=True, null=True),
        ),
        migrations.AddField(
            model_name="planpricing",
            name="created",
            field=models.DateTimeField(
                auto_now_add=True,
                db_index=True,
                default=None,
                null=True,
                verbose_name="created",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="planpricing",
            name="updated_at",
            field=models.DateTimeField(auto_now=True, null=True),
        ),
        migrations.AddField(
            model_name="planquota",
            name="created",
            field=models.DateTimeField(
                auto_now_add=True,
                db_index=True,
                default=None,
                null=True,
                verbose_name="created",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="planquota",
            name="updated_at",
            field=models.DateTimeField(auto_now=True, null=True),
        ),
        migrations.AddField(
            model_name="pricing",
            name="created",
            field=models.DateTimeField(
                auto_now_add=True,
                db_index=True,
                default=None,
                null=True,
                verbose_name="created",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="pricing",
            name="updated_at",
            field=models.DateTimeField(auto_now=True, null=True),
        ),
        migrations.AddField(
            model_name="quota",
            name="created",
            field=models.DateTimeField(
                auto_now_add=True,
                db_index=True,
                default=None,
                null=True,
                verbose_name="created",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="quota",
            name="updated_at",
            field=models.DateTimeField(auto_now=True, null=True),
        ),
        migrations.AddField(
            model_name="recurringuserplan",
            name="created",
            field=models.DateTimeField(
                auto_now_add=True,
                db_index=True,
                default=None,
                null=True,
                verbose_name="created",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="recurringuserplan",
            name="updated_at",
            field=models.DateTimeField(auto_now=True, null=True),
        ),
        migrations.AddField(
            model_name="userplan",
            name="created",
            field=models.DateTimeField(
                auto_now_add=True,
                db_index=True,
                default=None,
                null=True,
                verbose_name="created",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="userplan",
            name="updated_at",
            field=models.DateTimeField(auto_now=True, null=True),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="shipping_street",
            field=models.CharField(blank=True, max_length=200, verbose_name="Street"),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="shipping_zipcode",
            field=models.CharField(blank=True, max_length=200, verbose_name="Zip code"),
        ),
        migrations.AlterField(
            model_name="order",
            name="created",
            field=models.DateTimeField(
                auto_now_add=True, db_index=True, null=True, verbose_name="created"
            ),
        ),
        migrations.AlterField(
            model_name="plan",
            name="created",
            field=models.DateTimeField(
                auto_now_add=True, db_index=True, null=True, verbose_name="created"
            ),
        ),
    ]
