# Generated by Django 5.2.2 on 2025-06-06 05:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "plans",
            "0014_recurringuserplan_has_automatic_renewal_backup_deprecated_to_renewal_triggered_by",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="recurringuserplan",
            name="last_renewal_attempt",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="last renewal attempt"
            ),
        ),
    ]
